/*
 *    trida DBase
 *    programu Fosforuv DNS server v1.0
 *    Jan Pospisil, 2006
 *
 *    Trida uchovava DNS zaznamy, tj. ciselne adresy k jednotlivym
 *    domenovym jmenum. Zaroven umoznuje vyhledavani v techto zaznamech.
 *
 */


#ifndef __AF__DBASE_CPP_
#define __AF__DBASE_CPP_

#include <bitset.h>
#include <fstream>
#include <sstream>
#include <winsock>
#include "./DNSDomain.cpp"

#ifndef NULL
#define NULL 0
#endif

struct subdomain {
	string name;
	long ip;
	subdomain * next;
	subdomain * sub;
};

class DBase {
	subdomain * root;

	public:
		DBase();
		void add(string *, unsigned long);
		void add(DNSDomain *, unsigned long);
		bool loadFile(char *);
		long find(DNSDomain *);
		string showSubdomains(subdomain *, int);
		friend ostream & operator<<(ostream &, DBase &);
		friend ostream & operator<<(ostream &, DBase *);
		static string inet_addr_rev(unsigned long);
};

//////////////////////////////////////////////////////////////////////////////

DBase::DBase() {
	root = new subdomain();
	root->name = " ";
	root->ip = 0;
	root->next = NULL;
	root->sub = NULL;
}

void DBase::add(string * str, unsigned long ip) {
	DBase::add(new DNSDomain(str), ip);
}

void DBase::add(DNSDomain * domain, unsigned long ip) {
	int i = 1;

	subdomain * parent = root;
	subdomain * soused;
	while (parent) {
		soused = parent->sub;
		while (soused) {
			if (soused->name == domain->getLabel(i))
				break;
			soused = soused->next;
		}
		if (soused) {
			if (i == domain->getCount()-1) {
				soused->ip = ip;
				return;
			}
			parent = soused;
			i++;
		} else {
			subdomain * tmp = new subdomain();
			tmp->name = domain->getLabel(i);
			tmp->ip = 0;
			tmp->next = parent->sub;
			tmp->sub = NULL;
			parent->sub = tmp;
		}
	}
}

bool DBase::loadFile(char * filename) {
	ifstream in(filename);
	if (in.fail())
		return false;
	char buf[255];
	char buf2[255];
	while (in.getline(buf, 255)) {
		if (buf[0] != '#' && buf[0] != ' ' && buf[0] != '\0') {
			in.getline(buf2,255);
			DBase::add(new string(buf), inet_addr(buf2));
		}
	}
	in.close();
	return true;
}

long DBase::find(DNSDomain * domain) {
	int i = 1;

	subdomain * parent = DBase::root;
	subdomain * soused;
	while (parent) {
		soused = parent->sub;
		while (soused) {
			if (soused->name == domain->getLabel(i))
				break;
			soused = soused->next;
		}
		if (soused) {
			if (i == domain->getCount()-1) {
				return soused->ip;
			}
			parent = soused;
			i++;
		} else {
			return NULL;
		}
	}
	return NULL;
}

string DBase::showSubdomains(subdomain * dom, int level = 0) {
	ostringstream os;
	os << "   |   ";
	for (int i = 0; i < level; i++)
		os << "  ";
	os << dom->name;
	if (dom->ip > 0)
		os << " (" << DBase::inet_addr_rev(dom->ip) << ")";
	os << endl;
	subdomain * domain = dom->sub;
	while (domain) {
		os << DBase::showSubdomains(domain, level+1);
		domain = domain->next;
	}
	return os.str();
}

ostream & operator<<(ostream & o, DBase & db) {
	o << db.showSubdomains(db.root,0);
	return o;
}

ostream & operator<<(ostream & o, DBase * db) {
	o << db->showSubdomains(db->root,0);
	return o;
}

string & operator+=(string & str, unsigned long cislo) {
	char buf[100];
	sprintf(buf, "%d", cislo);
	str += buf;
	return str;
}

string DBase::inet_addr_rev(unsigned long ip) {
	string res;
	unsigned long cislo;
	cislo = ip % 256;
	ip = ip/256;
	res += cislo;
	res += ".";
	cislo = ip % 256;
	ip = ip/256;
	res += cislo;
	res += ".";
	cislo = ip % 256;
	ip = ip/256;
	res += cislo;
	res += ".";
	cislo = ip % 256;
	res += cislo;
	return res;
}

#endif /*__AF__DBASE_CPP_*/