/*
 *    trida Message
 *    programu Fosforuv DNS server v1.0
 *    Jan Pospisil, 2006
 *
 *    Jedna se o tridu zpracovavajici dotazy na odpovedi odpovidajici standardu DNS.
 */


#ifndef __AF_MESSAGE_CPP
#define __AF_MESSAGE_CPP

#include "./Dbase.cpp"
#include "./Sock.cpp"

#ifndef NULL
#define NULL 0
#endif

const int MAX_L = 512;

class Message {
	char * data;
	SockData * sdata;

	int id;
	bool qr;
	int opcode;
	bool aa, tc, rd, ra;
	int rcode;
	int qdcount, ancount, nscount, arcount;
	char qname[MAX_L];

	DBase * db;

	public:
		Message(DBase *);
		void load(SockData *);
		void makeRespons();

	private:
		void parse();
};

///////////////////////////////////////////////////////////////////////////////////////////////////

Message::Message(DBase * db) {
	Message::db = db;
}

void Message::load(SockData * data) {
	Message::data = data->data;
	sdata = data;
	parse();
}

void Message::parse() {
	id = MAKEWORD(data[1], data[0]);
	qr = data[2] & 1;
	opcode = (data[2] & 30)/2;
	aa = data[2] & 32;
	tc = data[2] & 64;
	rd = data[2] & 128;
	ra = data[3] & 1;
	rcode = (data[3] & 240)/16;
	qdcount = MAKEWORD(data[5], data[4]);
	ancount = MAKEWORD(data[7], data[6]);
	nscount = MAKEWORD(data[9], data[8]);
	arcount = MAKEWORD(data[11], data[10]);
	
	if (qdcount > 0) {
		int size = data[12];
		int j = 0;
		while (size) {
			for (int i = 0; j < MAX_L && i < size; i++, j++)
				qname[j] = data[j+13];
			size = data[j+13];
			if (size)
				qname[j++] = '.';
		}
		qname[j] = '\0';
	}
}

void Message::makeRespons() {
	long ip = db->find(new DNSDomain(new string(qname)));
	if (ip == 0) {
		// adresu nezname => udelat chybovou zpravu, popr presmerovat na jiny server
		// nebo se dotazat nekoho jineho a to poslat...
		return;
	}
	data[7] = 1;    // pocet odpovedi ve zprave
	data[sdata->size++] = 192;    //    jedna se o odpoved pro 
	data[sdata->size++] = 12;     //    tazanou adresu
	data[sdata->size++] = 0;
	data[sdata->size++] = 1;
	data[sdata->size++] = 0;
	data[sdata->size++] = 1;
	data[sdata->size++] = 0;
	data[sdata->size++] = 0;
	data[sdata->size++] = 7;
	data[sdata->size++] = 8;
	data[sdata->size++] = 0;
	data[sdata->size++] = 4;
	*((long*)&data[sdata->size]) = ip;
	sdata->size += 4;
}

#endif /*__AF_MESSAGE_CPP*/