/*
 *    trida Sock
 *    programu Fosforuv DNS server v1.0
 *    Jan Pospisil, 2006
 *
 *    Trida zajistuje sitovou komunikaci pomoci protokolu UDP v blokovacim rezimu.
 *
 */


#ifndef __AF_SOCK_CPP_
#define __AF_SOCK_CPP_

#include <winsock>

#ifndef NULL
#define NULL 0
#endif

#ifndef IP_LOCALHOST
#define IP_LOCALHOST 16777343   // 127.0.0.1
#endif

struct SockData {
	char * data;
	int size;
	int maxSize;
	sockaddr_in info;
};


class Sock {

	SOCKET s;

	public:
		Sock();
		~Sock();
		void listen() {listen(53, INADDR_ANY);}
		void listen(unsigned int port) {listen(port, INADDR_ANY);}
		void listens(unsigned int port, const char * ip) {listen(port, inet_addr(ip));}
		void listen(unsigned int, long);
		Sock & operator>>(SockData &);
		Sock & operator<<(SockData &);

	private:
			void chyba();

};

///////////////////////////////////////////////////////////////////////////////////

Sock::Sock() {
	WSADATA data;
	if (WSAStartup(MAKEWORD(1,1), &data) != 0)
		chyba();
}

Sock::~Sock() {
	closesocket(s);
	WSACleanup();
}

void Sock::listen(unsigned int port, long ip) {
			
	s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (s == INVALID_SOCKET)
		chyba();

	sockaddr_in server;
	server.sin_family = AF_INET;
	server.sin_port = htons(port);
	server.sin_addr.s_addr = ip;

	if (bind(s, (sockaddr *) & server, sizeof(server)) == SOCKET_ERROR)
		chyba();
}

Sock & Sock::operator>>(SockData & data) {
	int tmp = sizeof(data.info);

	data.size = recvfrom(s, data.data, data.size - 1, 0, (sockaddr *)&data.info, & tmp);

	if (data.size == SOCKET_ERROR)
		chyba();
	return *this;
}

Sock & Sock::operator<<(SockData & data) {
	if (data.size > 512)
		throw "Posilana data vetsi nez 512 b.";
	if (sendto(s, data.data, data.size, 0, (sockaddr*)&data.info, sizeof(data.info)) == SOCKET_ERROR)
		chyba();
	return *this;
}

void Sock::chyba() {
	int chyba = WSAGetLastError();
	WSACleanup();
	switch (chyba)
	{
		case WSAEINTR:			// 10004
			throw "Blokovaci operace stornovana. Firewall?";
		case WSAEINVAL:			// 10022
			throw "Soket neni svazan s adresou sitove karty a cislem portu. Nejprve musite zavolat funkci bind. Az pote listen.";
		case WSAEMFILE:			// 10024
			throw "Prilis mnoho otevrenych soketu.";
		case WSAEINPROGRESS:	// 10036
			throw "Zaroven probiha blokovaci operace.";
		case WSAENOTSOCK:		// 10038
			throw "Prvni parametr funkce listen neni identifikator soketu.";
		case WSAEMSGSIZE:		// 10040
			throw "Posilana data jsou prilis velika.";
		case WSAEOPNOTSUPP:		// 10045
			throw "Nelze zavolat funkci listen na dany typ soketu.";
		case WSAEADDRINUSE:		// 10048
			throw "Adresa nebo port jsou jiz pouzivany.";
		case WSAENETDOWN:		// 10050
			throw "Nepracuje sit.";
		case WSAENOBUFS:		// 10055
			throw "Nepodarilo se vytvorit frontu pozadavku. Neni dostatek mista pro buffer.";
		case WSAEISCONN:		// 10056
			throw "Soket je jiz spojen.";
		case WSANOTINITIALISED: // 10093
			throw "Nebyla inicializovana knihovna soketu pomoc WSAStartup.";
		default :
			throw "Nechapu co se stalo :-)";
    }
}


#endif /*__AF_SOCK_CPP_*/